# -*- coding: utf-8 -*-
"""File generated by generate_code() - 
WARNING! All changes made in this file will be lost!
"""

from ..Classes.import_all import *

load_switch = {
    "Data": Data,
    "Data1D": Data1D,
    "DataFreq": DataFreq,
    "DataLinspace": DataLinspace,
    "DataND": DataND,
    "DataPattern": DataPattern,
    "DataTime": DataTime,
    "Norm_affine": Norm_affine,
    "Norm_func": Norm_func,
    "Norm_indices": Norm_indices,
    "Norm_ref": Norm_ref,
    "Norm_vector": Norm_vector,
    "Normalization": Normalization,
    "RequestedAxis": RequestedAxis,
    "VectorField": VectorField,
}
